# -*- coding: utf-8 -*-

# Moin-comments - Blog like comments in MoinMoin
# Copyright (C) 2009 José Lopes

## This file is part of Moin-comments.
##
## Moin-comments is free software: you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
##
## Moin-comments is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with Moin-comments.  If not, see <http://www.gnu.org/licenses/>.

#
# José Lopes <jose.lopes@paxjulia.com>
# Helder Guerreiro <helder@paxjulia.com>
#
# $Id: Comments.py 47 2010-04-02 23:00:39Z hguerreiro $
#

"""
Comments Macro

This macro display the comments page.
It collects the comments files and displays its content.

Usage:
    <<Comments([page_name])>>
"""

# General imports:
import os
import glob

# MoinMoin imports
from MoinMoin.Page import Page

# Utils
from comment_utils import *

# Auxiliary functions:

def comment_html(request, comment):
    _ = request.getText
    return '''<table>
    <tr><td>%(name)s</td><td>%(comment_name)s</td></tr>
    <tr><td>%(time)s</td><td>%(comment_time)s</td></tr>
    <tr><td colspan=2>%(comment_text)s</td></tr>
    </table>''' % {
    'name': _('Name:'),
    'comment_name': comment['user_name'],
    'time': _('Time:'),
    'comment_time': comment['time'].strftime('%Y.%m.%d %H:%M'),
    'comment_text': '<p>'.join( comment['comment'].split('\n') ),
    }

def navbar(request, page_number, max_pages, page_uri):
    _ = request.getText

    if max_pages == 1:
        return ''

    html = ['<div class="navbar">']
    if page_number > 1:
        html.append('<div class="prevcmt">')
        html.append('<a href="%s">%s</a>&nbsp;&nbsp;' %
                (page_uri,_('|&lt;')))
        html.append('<a href="%s?page_number=%d">%s</a>&nbsp;&nbsp;' %
                (page_uri,page_number-1,_('&lt;&lt;')))
        html.append('</div>')

    if page_number < max_pages:
        html.append('<div class="nextcmt">')
        html.append('<a href="%s?page_number=%d">%s</a>&nbsp;&nbsp;' %
                (page_uri,page_number+1,_('&gt;&gt;')))
        html.append('<a href="%s?page_number=%d">%s</a>&nbsp;&nbsp;' %
                (page_uri,max_pages,_('&gt;|')))
        html.append('</div>')

    html.append('</div>')

    return '\n'.join(html)

def macro_Comments(macro, page_name=u''):
    '''
    Usage:

        <<Comments(page_name)>>
        Shows the comments of page 'page_name'
    or
        <<Comments()>>
        Shows the page of the current page.
    '''
    _ = macro.request.getText
    request = macro.request
    formatter = macro.formatter


    # By default show the comments for the current page
    if page_name == u'':
        page_name = macro.formatter.page.page_name

    # Get the configuration:
    page = Page(request, page_name )
    comments_dir = page.getPagePath("comments", check_create=1)

    # Get the page_name comment list
    files = glob.glob(os.path.join(comments_dir,'*.txt'))
    files.sort()

    # Compose the comments markup
    html = [u'<a name="comment_section"></a>']
    if not files:
        html.append(u'<p>%s</p>' % _('There are no comments'))
    else:
        # Pagination
        cmt_per_page = get_cfg_int(macro, 'comment_cmt_per_page',50)

        if cmt_per_page:
            page_uri = request.url.split('?')[0]

            number_messages = len(files)
            if number_messages % cmt_per_page:
                offset = 1
            else:
                offset = 0
            max_pages = number_messages / cmt_per_page + offset
            try:
                page_number = get_input_int(macro, 'page_number', 1 )
            except ValueError:
                page_number = 1
            if page_number > max_pages:
                page_number = max_pages
            elif page_number < 1:
                page_number = 1

            first = (page_number - 1) * cmt_per_page
            last  = first + cmt_per_page

            files = files[first:last]

        # Get the comments contents
        comments = [ read_comment(Xi) for Xi in files]

        for comment in comments:
            html.append( u"%s" % comment_html(request, comment ) )

        if cmt_per_page:
            html.append(navbar(request, page_number, max_pages, page_uri))

    try:
        return formatter.rawHTML('\n'.join(html))
    except:
        return formatter.escapedText('')



# -*- coding: utf-8 -*-

# Moin-comments - Blog like comments in MoinMoin
# Copyright (C) 2009 José Lopes

## This file is part of Moin-comments.
##
## Moin-comments is free software: you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
##
## Moin-comments is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with Moin-comments.  If not, see <http://www.gnu.org/licenses/>.

#
# José Lopes <jose.lopes@paxjulia.com>
# Helder Guerreiro <helder@paxjulia.com>
#
# $Id: AddComment.py 42 2010-04-02 18:29:13Z hguerreiro $
#


"""
Add Comment Macro

This macro places the comment form on the page.

Usage:
    <<AddComment(Add a Comment title, Add button label)>>
"""

# MoinMoin imports:
from MoinMoin import wikiutil
from MoinMoin.Page import Page
from MoinMoin.mail import sendmail
from MoinMoin.datastruct.backends import GroupDoesNotExistError

from datetime import datetime
from random import choice
from string import letters, digits

import os

from comment_utils import get_cfg, get_input, write_comment, notify_subscribers

# Auxiliary class:
class AddComment:
    def __init__(self, macro ):
        self.macro = macro
        self.page = macro.request.page
        self.user = macro.request.user
        self.page_name = macro.formatter.page.page_name

        self.msg = ''
        self.reset_comment()
        self.errors = []

        try:
            passpartout_group = macro.request.groups[
                get_cfg(macro, 'comment_passpartout_group', 'PasspartoutGroup' )]

            if self.user.name in passpartout_group:
                passpartout = True
            else:
                passpartout = False
        except GroupDoesNotExistError:
            passpartout = False

        self.passpartout = passpartout
        self.moderate = get_cfg(macro, 'comment_moderate', True) and not passpartout

        if macro.request.method == 'POST':
            self.save_comment()

    def reset_comment(self):
        '''Resets the comment dict to default a value'''
        self.comment = {
                'user_name' : self.user.name,
                'comment': '',
                'email': '',
                }

    def get_comment(self):
        self.comment = {
            'user_name' : get_input(self.macro, 'user_name'),
            'comment': get_input(self.macro, 'comment'),
            'email': get_input(self.macro, 'email'),
            }

    def errors_check(self):
        """
        Check the form for errors.
        """
        _ = self.macro.request.getText

        errors = []

        if not self.comment['user_name']:
            errors.append( _('You must enter your name.') )
        if len(self.comment['user_name']) > 128:
            errors.append( _('Please use a shorter name.') )
        if not self.comment['comment']:
            errors.append( _('You have yet to write your comment.') )
        if len(self.comment['comment']) > 10240:
            errors.append( _('Maximum number of characters is 10240.'))

        if ( get_cfg(self.macro, 'comment_recaptcha', False) and not self.passpartout
            and not self.captcha.is_valid):
            errors.append( _("I'm not sure you're human! Please fill in the captcha."))

        return errors

    def save_comment( self ):
        _ = self.macro.request.getText

        if get_input(self.macro, 'do' ) != u'comment_add':
            # This is not a comment post do nothing
            return

        if get_cfg(self.macro, 'comment_recaptcha', False ) and not self.passpartout:
            import captcha
            self.captcha = captcha.submit (
                get_input(self.macro, 'recaptcha_challenge_field'),
                get_input(self.macro, 'recaptcha_response_field'),
                get_cfg(self.macro, 'comment_recaptcha_private_key'),
                self.macro.request.remote_addr )

        self.get_comment()
        self.errors = self.errors_check()

        if not self.errors: # Save the comment
            # Find out where to save the comment:
            if self.moderate:
                # This commet will be added to the moderation queue
                page = Page(self.macro.request,
                    get_cfg(self.macro, 'comment_approval_page', 'CommentsApproval'))
                comment_dir = page.getPagePath('', check_create=0)
            else:
                # The comment will be immediately posted
                page = Page(self.macro.request,self.page_name)
                comment_dir = page.getPagePath('comments', check_create=1)

            # Compose the comment structure and write it
            now = datetime.now()
            random_str =  ''.join([choice(letters + digits) for i in range(20)])
            comment_file = '%s-%s.txt' % (now.strftime("%s"), random_str)
            file_name = os.path.join(comment_dir, comment_file)

            comment = self.comment
            comment['page'] = self.page_name
            comment['time'] = now
            if get_cfg(self.macro, 'comment_store_addr', False):
                comment['remote_addr'] = self.macro.request.remote_addr

            if self.moderate:
                self.msg = _('Your comment awaits moderation. Thank you.')
            else:
                self.msg = _('Your comment has been posted. Thank you.')

            write_comment( file_name, comment )

            if self.moderate:
                # If we have defined a list of moderators to notify and this user is
                # moderated then a message is sent to the moderator list
                moderators = get_cfg(self.macro, 'comment_moderators', None)
                if moderators:
                    sendmail.sendmail( self.macro.request, moderators.split(','),
                    _('New comment awaits moderation for page %(page)s' % self.comment ),
                    _('New comment awaits moderation:\n\nPage: %(page)s\nFrom: %(user_name)s\nMessage:\n\n%(comment)s\n\n--' %
                        self.comment ))
            else:
                # Send notification to page subscribers if the page
                notify_subscribers(self.macro, self.comment)

            # clean up the fields to display
            self.reset_comment()

    def renderInPage(self):
        """
        Render comments form in page context.
        """
        _ = self.macro.request.getText
        html = u'''<div class="comments_form">
        <form method="POST" action="%(page_uri)s">
        <input type="hidden" name="do" value="comment_add">
        <table>''' % { 'page_uri': self.macro.request.request.url }

        html += '''
            <tr>
                <td colspan=2 id="center_cell"><b>%(header)s</b></td>
            </tr>
            <tr>
                <th>%(name_label)s</th>
                <td>
                    <input type="text" id="name" maxlength=128 name="user_name"
                           value="%(user_name)s">
                </td>
            </tr>
            <tr>
                <th>%(comment_label)s</th>
                <td>
                    <textarea name="comment">%(comment)s</textarea>
                </td>
            </tr>
            ''' % {
            'page_name': self.page_name,
            'user_name': self.comment['user_name'],
            'comment':   self.comment['comment'],
            'header': _('Comment this page'),
            'name_label': _('Name:'),
            'comment_label': _('Comment:')  }

        if self.msg:
            html += u'<tr><td colspan = 2><div id="comment_message">'
            html += u'<p>%s</p>' % self.msg
            html += u'</div></td></tr>'

        if self.errors:
            html += u'<tr><td colspan = 2><div id="comment_error">'
            if len(self.errors) > 1:
                html += u'<p>%s</p><ul>'  % _('Your comment has errors:')
            else:
                html += u'<p>%s</p><ul>'  % _('Your comment has one error:')
            for error in self.errors:
                html += u'<li>%s</li>' % error
            html += u'</ul></div></td></tr>'

        if get_cfg(self.macro, 'comment_recaptcha', False) and not self.passpartout:
            import captcha
            html += u"""
            <tr>
                <th>%(recaptcha_label)s</th>
                <td>%(recaptcha)s</td>
            </tr>""" % {
            'recaptcha' : captcha.displayhtml(
                                get_cfg(self.macro, 'comment_recaptcha_public_key')),
            'recaptcha_label': _('Are you human?') }

        html += """
             <tr>
                <td colspan=2 id="center_cell"><input type="submit" value="%(label)s">
                </td>
            </tr>
        </table></form></div>""" % { 'label': _('Send comment') }

        try:
            return self.macro.formatter.rawHTML(html)
        except:
            return self.macro.formatter.escapedText('')


# Macro function:
def macro_AddComment(macro):
    return AddComment(macro).renderInPage()



# -*- coding: utf-8 -*-

# Moin-comments - Blog like comments in MoinMoin
# Copyright (C) 2009 José Lopes

## This file is part of Moin-comments.
##
## Moin-comments is free software: you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
##
## Moin-comments is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with Moin-comments.  If not, see <http://www.gnu.org/licenses/>.

#
# José Lopes <jose.lopes@paxjulia.com>
#
# $Id: ApproveComments.py 43 2010-04-02 19:03:55Z hguerreiro $
#

"""
Approve Comments Macro

This macro lists the comments waiting for approval and displays the
delete and approval buttons.

Usage:
    <<ApproveComments()>>

Requirements:
    You have to create a page where the comments are stored for moderation. This
    page is defined with the configuration variable:

        comment_approval_page

    And by default has a value of:

        CommentsApproval
"""

# General imports:
from MoinMoin.Page import Page
from MoinMoin import wikiutil

import os
from datetime import datetime
import glob

from comment_utils import get_cfg, get_input, read_comment, notify_subscribers

class ApproveError(Exception): pass


class ApproveComments:
    def __init__(self, macro ):
        self.macro = macro
        self.page_name = get_cfg(macro, 'comment_approval_page',
                              'CommentsApproval')
        self.msg = []

        if self.page_name != macro.formatter.page.page_name:
            # It's mandatory to run the ApproveComments macro from the
            # comment_approval_page defined in the configuration
            return

        page = Page(macro.request,self.page_name)
        if not page.exists():
            raise ApproveError('You have to create the approval page!')
        self.approval_dir = page.getPagePath('', check_create=0)

        if macro.request.method == 'POST':
            if get_input( macro, 'do' ) == u'comment_delete':
                self.delete_comment()
            if get_input( macro, 'do' ) == u'comment_approve':
                self.approve_comment()

    def delete_comment(self):
        _ = self.macro.request.getText

        file_name = get_input( self.macro, 'file' )
        os.remove(os.path.join(self.approval_dir, file_name))
        self.msg.append(_('Comment deleted'))

    def approve_comment(self):
        _ = self.macro.request.getText

        # Source
        origin = os.path.join(self.approval_dir, get_input(self.macro, 'file'))
        comment = read_comment( origin )

        # Destination
        page = Page(self.macro.request, comment['page'] )
        if not page.exists():
            self.msg.append(_('The page this comment was written for don\'t exist any more'))
            return

        dest_dir = page.getPagePath("comments", check_create=1)
        destination = os.path.join(dest_dir,get_input(self.macro, 'file'))

        # Rename the file:
        os.rename(origin, destination)
        self.msg.append(_('Comment approved'))

        # Notify page subscribers:
        notify_subscribers(self.macro, comment)

    def render_in_page(self):

        def cmp_page_time( a, b ):
            if a['page'] < b['page']:
                return -1
            elif a['page'] > b['page']:
                return 1
            else:
                if a['time'] < b['time']:
                    return -1
                elif a['time'] > b['time']:
                    return 1
            return 0

        _ = self.macro.request.getText

        if self.page_name != self.macro.formatter.page.page_name:
            return self.macro.formatter.text(
             _('Sorry, but the  ApproveComments macro must be '
               'used on %(page_name)s page.' %
                { 'page_name': self.page_name } ))

        html = []

        if self.msg:
            html.append('<div class="comment_message"><ul>')
            for m in self.msg:
                 html.append('<li>%s</li>' % m)
            html.append('</ul></div>')

        files = glob.glob(os.path.join(self.approval_dir,'*.txt'))
        if not files:
            html.append(u'<p>%s</p>' % _("There's no comment awaiting for moderation."))
        else:
            comments = []

            # Read the comments:
            for file_name in files:
                comment = read_comment( file_name )
                comment['file_name'] = file_name
                comments.append(comment)

            # Sort the coments by page, then by time
            comments.sort(cmp_page_time)

            for comment in comments:
                html.append( u'''<div class="comment_approval">
<table>
  <tr><th colspan=2>%(intro)s %(page_name)s</th></tr>
  <tr><td>%(name)s</td><td>%(comment_name)s</td></tr>
  <tr><td>%(time)s</td><td>%(comment_time)s</td></tr>''' % {
                    'intro': _('Comment to'),
                    'page_name': comment['page'],
                    'name': _('Name:'),
                    'time': _('Time:'),
                    'comment_time': comment['time'].strftime('%Y.%m.%d %H:%M'),
                    'comment_name': comment['user_name'],
                    })
                if get_cfg(self.macro, 'comment_store_addr', False):
                    html.append(u'  <tr><td>%(remote_addr)s</td><td>%(comment_addr)s</td></tr>' % {
                    'remote_addr': _('Remote address:'),
                    'comment_addr': comment['remote_addr'],
                    })
                html.append(u'''  <tr><td colspan=2>%(comment_text)s</td></tr>
  <tr>
    <td colspan=2>
      <form method="POST" action="%(page_uri)s">
        <input type="hidden" name="do" value="comment_delete">
        <input type="submit" value="%(button_delete)s" id="delete">
        <input type="hidden" name="file" value="%(comment_file)s">
      </form>
      <form method="POST" action="%(page_uri)s">
        <input type="hidden" name="do" value="comment_approve">
        <input type="submit" value="%(button_accept)s" id="ok">
        <input type="hidden" name="file" value="%(comment_file)s">
      </form>
    </td>
  </tr>
</table>
</div><br />''' % {
                'comment_text': '<p>'.join( comment['comment'].split('\n') ),
                'comment_file': os.path.basename(comment['file_name']),
                'page_uri': self.macro.request.request.url,
                'button_delete': _('Delete'),
                'button_accept': _('Accept'),
                } )

        try:
            return self.macro.formatter.rawHTML('\n'.join(html))
        except:
            return self.macro.formatter.escapedText('')

# Macro function:
def macro_ApproveComments(macro):
    return ApproveComments(macro).render_in_page()
